﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Serialization;
using System.IO;

namespace lwftp
{


    public class lwsetting
    {
        public string name { get; set; }
        public string ftpAddr { get; set; }
        public string ftpUsername { get; set; }
        public string ftpPassword { get; set; }
        public string fileMask { get; set; }
        public bool ftpPassive { get; set; }
        public Int32 numFiles { get; set; }
        public string outputPath { get; set; }

        [XmlIgnore]
        public lwsettings parent { get; set; }


        public lwsetting()
        {
            this.name = "";
            this.ftpAddr = "";
            this.ftpUsername = "";
            this.ftpPassword = "";
            this.fileMask = "";
            this.ftpPassive = true;
            this.numFiles = 1;
            this.outputPath = @"\";
            this.parent = null;

        }
    }


    public class templateList : List<lwsetting>
    {
        public lwsetting Find(string name)
        {

            foreach (lwsetting template in this)
            {


                if (template.name.Equals(name, StringComparison.CurrentCultureIgnoreCase))
                {
                    return template;
                }


            }

            return null;
        }
    }

    public class lwsettings
    {
        public templateList templates = new templateList();
        public Int32 maxFiles { get; set; }
        public bool cleanOld { get; set; }
        public bool useProxy { get; set; }
        public bool proxyUseCurrent { get; set; }
        public string proxyAddr { get; set; }
        public string proxyUsername { get; set; }
        public string proxyPassword { get; set; }
        public string proxyDomain { get; set; }
        public string[] proxyExclude { get; set; }
        public string serverRegex { get; set; }
        public Int32 blockSize { get; set; }

        public lwsettings()
        {
            this.maxFiles = 1;
            this.cleanOld = false;
            this.useProxy = false;
            this.proxyUseCurrent = true;
            this.proxyAddr = "";
            this.proxyUsername = "";
            this.proxyPassword = "";
            this.serverRegex = @"^(?<dir>[\-ld])(?<permission>([\-r][\-w][\-xs]){3})\s+(?<filecode>\d+)\s+(?<owner>\w+)\s+(?<group>\w+)\s+(?<size>\d+)\s+(?<timestamp>((?<month>\w{3})\s+(?<day>\d{2})\s+(?<hour>\d{1,2}):(?<minute>\d{2}))|((?<month>\w{3})\s+(?<day>\d{1,2})\s+(?<year>\d{4})))\s+(?<name>.+)$";
            this.proxyExclude = new string[] { "127.0.0.1", "localhost" };
            this.blockSize = 65536;
        }

        public void Add(lwsetting template)
        {
            template.parent = this;
            templates.Add(template);
        }

        public bool toFile(string name)
        {
            try
            {
                XmlSerializer serializer = new XmlSerializer(typeof(lwsettings));
                FileStream fs = new FileStream(name, FileMode.Create);
                serializer.Serialize(fs, this);
                fs.Close();
                return true;
            }
            catch
            {
                return false;
            }

        }


        public static lwsettings fromFile(string name)
        {
            lwsettings settings;
            XmlSerializer serializer = new XmlSerializer(typeof(lwsettings));

            try
            {
                FileStream fs = new FileStream(name, FileMode.Open);
                settings = serializer.Deserialize(fs) as lwsettings;
                settings.templates.ForEach(temp => temp.parent = settings);
            }
            catch
            {
                settings = null;
            }
            return settings;
        }

        IEnumerator<lwsetting> GetEnumerator()
        {

            foreach (lwsetting template in templates)
            {
                yield return template;
            }
        }
    }



}
