﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Text.RegularExpressions;
using System.Net;
using System.Globalization;
using System.IO;


namespace lwftp
{

    public class fileTransfer
    {

        protected bool stopped;
        protected lwsetting template;
        protected EventLog events;

        public fileTransfer(lwsetting withTemplate, EventLog withEvents)
        {
            stopped = false;
            template = withTemplate;
            events = withEvents;
        }

        public int Process()
        {


            FtpWebRequest ListRequest = System.Net.FtpWebRequest.Create(template.ftpAddr) as FtpWebRequest;
            ListRequest.Credentials = new NetworkCredential(template.ftpUsername, template.ftpPassword);
            ListRequest.Method = WebRequestMethods.Ftp.ListDirectoryDetails;
            ListRequest.UsePassive = template.ftpPassive;
            ListRequest.UseBinary = false;

            if (template.parent.useProxy)
            {
                if (template.parent.proxyUseCurrent)
                    ListRequest.Proxy = new WebProxy(template.parent.proxyAddr, true, template.parent.proxyExclude, CredentialCache.DefaultCredentials);
                else
                    ListRequest.Proxy = new WebProxy(template.parent.proxyAddr, true, template.parent.proxyExclude, new NetworkCredential(template.parent.proxyUsername, template.parent.proxyPassword, template.parent.proxyDomain));

            }




            FtpWebResponse ListResponse;

            try
            {
                ListResponse = ListRequest.GetResponse() as FtpWebResponse;
            }

            catch (Exception e)
            {
                ListResponse = null;
                events.addEvent(String.Format("Error on getting response (directory listing) {0}", e.Message));
            }




            if (ListResponse != null)
            {

                String SingleLineDirectoryList;

                try
                {

                    using (StreamReader ListStreamReader = new StreamReader(ListResponse.GetResponseStream()))
                    {
                        SingleLineDirectoryList = ListStreamReader.ReadToEnd();
                    }


                }

                catch (Exception e)
                {
                    events.addEvent(String.Format("Error on getting response stream (directory listing) {0}", e.Message));
                    SingleLineDirectoryList = "";
                    stopped = true;
                }
                finally
                {
                    if (ListResponse != null)
                    {
                        ListResponse.Close();
                    }

                }


                if (stopped) return -1;

                List<string> DirectoryStrings = new List<string>(SingleLineDirectoryList.Split(new string[] { "\n" }, StringSplitOptions.RemoveEmptyEntries));



                /*     Regex ftpUnixListInfo = new Regex(@"^((?<DIR>([dD]{1}))|)(?<ATTRIBS>(.*))\s(?<SIZE>([0-9]{1,}))\s(?<DATE>((?<MONTHDAY>((?<MONTH>(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec))\s(?<DAY>([0-9\s]{2}))))\s(\s(?<YEAR>([0-9]{4}))|(?<TIME>([0-9]{2}\:[0-9]{2})))))\s(?<NAME>([A-Za-z0-9\-\._\s]{1,}))$",
                            RegexOptions.IgnoreCase | RegexOptions.CultureInvariant | RegexOptions.IgnorePatternWhitespace
                            | RegexOptions.Compiled);
                */

                Regex ftpUnixListInfo = new Regex(template.parent.serverRegex != "" ? template.parent.serverRegex : @"^(?<dir>[\-ld])(?<permission>([\-r][\-w][\-xs]){3})\s+(?<filecode>\d+)\s+(?<owner>\w+)\s+(?<group>\w+)\s+(?<size>\d+)\s+(?<timestamp>((?<month>\w{3})\s+(?<day>\d{2})\s+(?<hour>\d{1,2}):(?<minute>\d{2}))|((?<month>\w{3})\s+(?<day>\d{1,2})\s+(?<year>\d{4})))\s+(?<name>.+)$",
                                    RegexOptions.IgnoreCase | RegexOptions.CultureInvariant | RegexOptions.IgnorePatternWhitespace
                                   | RegexOptions.Compiled);


                ftpitems FtpItems = new ftpitems();

                foreach (string s in DirectoryStrings)
                {
                    Match FtpMatch = ftpUnixListInfo.Match(s);

                    if (FtpMatch.Groups["dir"].Value == "-" && Regex.IsMatch(FtpMatch.Groups["name"].Value, template.fileMask))
                    {

                        Int32 sz = Convert.ToInt32(FtpMatch.Groups["size"].Value);

                        string timestamp = FtpMatch.Groups["timestamp"].Value;
                        DateTime FtpFileTimeStamp;

                        if (!DateTime.TryParse(timestamp, new CultureInfo("en-US"), System.Globalization.DateTimeStyles.AssumeUniversal, out FtpFileTimeStamp))
                        {
                            timestamp = String.Format("{0} {1} {2} {3}:{4}", FtpMatch.Groups["day"].Value, FtpMatch.Groups["month"].Value, FtpMatch.Groups["year"].Value == "" ? DateTime.Now.Year.ToString() : FtpMatch.Groups["year"].Value, FtpMatch.Groups["hour"].Value, FtpMatch.Groups["minute"].Value);

                            if (!DateTime.TryParse(timestamp, new CultureInfo("en-US"), System.Globalization.DateTimeStyles.AssumeUniversal, out FtpFileTimeStamp))
                            {
                                FtpFileTimeStamp = DateTime.MinValue;
                            }

                        }

                        ftpitem OneItem = new ftpitem();
                        OneItem.filename = FtpMatch.Groups["name"].Value;
                        OneItem.path = template.ftpAddr.TrimEnd('/') + '/' + OneItem.filename;
                        OneItem.size = sz;
                        OneItem.timestamp = FtpFileTimeStamp;
                        //it.itemtype = ftpitem.Eitemtype.file;
                        FtpItems.Add(OneItem);

                    }

                }


                Int32 loaded = 0;
                Int32 blockSize = template.parent.blockSize;

                byte[] buffer = new byte[blockSize];


                string outpath = AppDomain.CurrentDomain.BaseDirectory + @"\" + template.outputPath.TrimEnd('\\') + "\\";
                Directory.CreateDirectory(outpath);

                foreach (ftpitem FtpFileInfo in (from item in FtpItems orderby item.timestamp descending select item))
                {
                    loaded++;

                    FtpWebRequest DownloadRequest = System.Net.FtpWebRequest.Create(FtpFileInfo.path) as FtpWebRequest;
                    DownloadRequest.Credentials = new NetworkCredential(template.ftpUsername, template.ftpPassword);
                    DownloadRequest.Method = WebRequestMethods.Ftp.DownloadFile;
                    DownloadRequest.UsePassive = template.ftpPassive;
                    DownloadRequest.UseBinary = true;
                    if (template.parent.useProxy)
                    {
                        if (template.parent.proxyUseCurrent)
                            DownloadRequest.Proxy = new WebProxy(template.parent.proxyAddr, true, template.parent.proxyExclude, CredentialCache.DefaultCredentials);
                        else
                            DownloadRequest.Proxy = new WebProxy(template.parent.proxyAddr, true, template.parent.proxyExclude, new NetworkCredential(template.parent.proxyUsername, template.parent.proxyPassword, template.parent.proxyDomain));

                    }

                    FtpWebResponse DownloadResponse;

                    try
                    {
                        DownloadResponse = DownloadRequest.GetResponse() as FtpWebResponse;
                    }


                    catch (Exception e)
                    {
                        events.addEvent(String.Format("Error on getting response (file download) {0}", e.Message));
                        DownloadResponse = null;
                    }




                    if (DownloadResponse != null)
                    {

                        stopped = false;

                        Display Disp = new Display(new string[] { String.Format("Loading file {0}", FtpFileInfo.filename), String.Format("Size {0}", FtpFileInfo.size), String.Format("Getting now {0} of {1}", loaded, template.numFiles), "" });

                        Int32 wr, maximum = FtpFileInfo.size, done = 0;


                        Stream DownloadResponseStream;
                        try
                        {
                            DownloadResponseStream = DownloadResponse.GetResponseStream();
                        }

                        catch (Exception e)
                        {
                            events.addEvent(String.Format("Error on getting response stream (download) {0}", e.Message));
                            DownloadResponseStream = null;
                            stopped = true;

                        }


                        FileStream TargetFileStream; string outFileName = outpath + FtpFileInfo.filename.Trim();

                        try
                        {
                            TargetFileStream = new FileStream(outFileName, FileMode.Create);
                        }
                        catch (Exception e)
                        {
                            events.addEvent(String.Format("Error creating output file (download) {0}", e.Message));
                            TargetFileStream = null;
                            stopped = true;

                        }


                        if (DownloadResponseStream != null && TargetFileStream != null && stopped == false)
                        {


                            try
                            {
                                while ((wr = DownloadResponseStream.Read(buffer, 0, blockSize)) != 0 && !stopped)
                                {
                                    TargetFileStream.Write(buffer, 0, wr); done += wr;
                                    Disp.RenderConsoleProgress((int)(((double)done / (double)maximum * 100)));

                                }
                            }
                            catch (Exception e)
                            {
                                events.addEvent(String.Format(@"Error on reading/writtng stream {0}", e.Message));
                                stopped = true;

                            }

                        }


                        if (DownloadResponseStream != null && !stopped) DownloadResponseStream.Dispose();
                        if (TargetFileStream != null) TargetFileStream.Close();
                        if (stopped && File.Exists(outFileName)) File.Delete(outFileName);
                        else File.SetCreationTime(outFileName, FtpFileInfo.timestamp);
                        if (DownloadResponse != null && !stopped) DownloadResponse.Close();
                    }
                    else
                    {
                        events.addEvent(String.Format("Null response on file download"));
                        return -1;

                    }


                    if (loaded == template.numFiles || stopped) { break; }

                }

            }
            else
            {
                events.addEvent(String.Format("Null response on file list"));
                return -1;

            }



            if (stopped) return -1; else return 0;
        }

    }
}
